#include "log.h"

extern PrintIP pip;



/****************************************************************************
 *
 * Function: OpenLogFile()
 *
 * Purpose: Create the log directory and file to put the packet log into
 *
 * Arguments: None.
 *
 * Returns: 0 on success, exits on error
 *
 ****************************************************************************/
int OpenLogFile()
{
   char log_path[STD_BUF];
   char log_file[STD_BUF];
   char proto[5];


   bzero(log_path, STD_BUF);
   bzero(log_file, STD_BUF);
   bzero(proto, 5);

   if((net.dip & pv.netmask) == pv.homenet)
   {

      if((net.sip & pv.netmask) != pv.homenet)
      {
         sprintf(log_path, "%s/%s", pv.log_dir, pip.saddr);
      }
      else
      {
         if( net.sp >= net.dp )
         {
            sprintf(log_path, "%s/%s", pv.log_dir, pip.saddr);
         }
         else
         {
            sprintf(log_path, "%s/%s", pv.log_dir, pip.daddr);
         }
      }
   }
   else
   {
      if((net.sip & pv.netmask) == pv.homenet)
      {
         sprintf(log_path, "%s/%s", pv.log_dir, pip.daddr);
      }
      else
      {
         if(net.sp >= net.dp)
         {
            sprintf(log_path, "%s/%s", pv.log_dir, pip.saddr);
         }
         else
         {
            sprintf(log_path, "%s/%s", pv.log_dir, pip.daddr);
         }
      }
   }

#ifdef DEBUG
   fprintf(stderr, "Creating directory: %s\n",log_path);
#endif

   if(mkdir(log_path,S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH))
   {
#ifdef DEBUG
      if(errno != EEXIST)
      {
         printf("Problem creating directory %s\n",log_path);
      }
#endif
   }

#ifdef DEBUG
   printf("Directory Created!\n");
#endif

   if((!strcasecmp(pip.proto, "TCP"))||
      (!strcasecmp(pip.proto, "UDP")))
   {
      if(pip.sport >= pip.dport)
      {
         sprintf(log_file, "%s/%s:%d-%d", log_path, pip.proto, pip.sport,
                 pip.dport);
      }
      else
      {
         sprintf(log_file, "%s/%s:%d-%d", log_path, pip.proto, pip.dport,
                 pip.sport);
      }
   }
   else
   {
      sprintf(log_file, "%s/%s", log_path, pip.proto);
   }

#ifdef DEBUG
   printf("Opening file: %s\n", log_file);
#endif

   if((log_ptr = fopen(log_file, "a")) == NULL)
   {
       fprintf(stderr, "ERROR: OpenLogFile() => fopen() log file: %s\n",
               strerror(errno));
       exit(1);
   }

#ifdef DEBUG
   printf("File opened...\n");
#endif

   return 0;
}



/****************************************************************************
 *
 * Function: PrintNetData(FILE *, char *,int)
 *
 * Purpose: Do a side by side dump of a buffer, hex dump of buffer bytes on
 *          the left, decoded ASCII on the right.
 *
 * Arguments: fp => ptr to stream to print to
 *            start => pointer to buffer data
 *            len => length of data buffer
 *
 * Returns: void function
 *
 ****************************************************************************/
void PrintNetData(FILE *fp, char *start, int len)
{
   char *end;              /* ptr to buffer end */
   char hexbuf[STD_BUF];   /* buffer for hex chars */
   char charbuf[STD_BUF];  /* buffer for decoded data */
   int col;                /* column incrementor */
   int i;                  /* counter */


   end = start + len; /* set the end of buffer ptr */

   do
   {
      col = 0;
      bzero(hexbuf,STD_BUF);
      bzero(charbuf,STD_BUF);

      for(i=0;i<16;i++)
      {
         if(start < end)
         {
            sprintf(hexbuf+(i*3),"%.2X ",start[0] & 0xFF);

            if(*start > 0x1F && *start < 0x7E)
            {
               sprintf(charbuf+i+col,"%c",start[0]);
            }
            else
            {
               sprintf(charbuf+i+col, ".");
            }
            start++;
         }
      }

      fprintf(fp,"     %-48s %s\n",hexbuf,charbuf);
      fflush(fp);

   }while(start < end);

   return;
}



/****************************************************************************
 *
 * Function: PrintIPPkt(FILE *, int)
 *
 * Purpose: Display the packet date to the stream pointer
 *
 * Arguments: fp => pointer to print data to
 *            type => packet protocol
 *
 * Returns: void function
 *
 ****************************************************************************/
void PrintIPPkt(FILE *fp, int type)
{
#ifdef DEBUG
   printf("PrintIPPkt type = %d\n", type);
#endif

   switch(type)
   {
      case IPPROTO_TCP:
                if(flow == RIGHT)
                {
                   fprintf(fp, "%s: %s %s:%d -> %s:%d ", pip.timestamp,
                           pip.proto,pip.saddr, pip.sport, pip.daddr,
                           pip.dport);
                }
                else
                {
                   fprintf(fp, "%s: %s %s:%d <- %s:%d ", pip.timestamp,
                           pip.proto,pip.daddr, pip.dport, pip.saddr,
                           pip.sport);
                }

                if(pip.flags & TH_SYN) fprintf(fp, "S"); else fprintf(fp, "*");
                if(pip.flags & TH_FIN) fprintf(fp, "F"); else fprintf(fp, "*");
                if(pip.flags & TH_RST) fprintf(fp, "R"); else fprintf(fp, "*");
                if(pip.flags & TH_PUSH) fprintf(fp, "P"); else fprintf(fp, "*");
                if(pip.flags & TH_ACK) fprintf(fp, "A"); else fprintf(fp, "*");
                if(pip.flags & TH_URG) fprintf(fp, "U"); else fprintf(fp, "*");

                fprintf(fp, "\n");

                fprintf(fp, "        %lX:%lX win=%lX TTL=%d\n",pip.seq, pip.ack,
                        pip.win, pip.ttl);
                break;

      case IPPROTO_UDP:
                if(flow == RIGHT)
                {
                   fprintf(fp, "%s: %s %s.%d -> %s.%d len=%d TTL=%d\n",
                           pip.timestamp, pip.proto, pip.saddr, pip.sport,
                           pip.daddr, pip.dport, pip.udp_len, pip.ttl);
                }
                else
                {
                   fprintf(fp, "%s: %s %s:%d <- %s:%d len=%d TTL=%d\n",
                           pip.timestamp, pip.proto, pip.daddr, pip.dport,
                           pip.saddr, pip.sport, pip.udp_len, pip.ttl);
                }

                break;

      case IPPROTO_ICMP:
                if(flow == RIGHT)
                {
                   fprintf(fp, "%s: %s %s -> %s\n          TTL=%d id=%d seq=%d %s\n", 
                           pip.timestamp, pip.proto, pip.saddr, pip.daddr, pip.ttl,
                           pip.icmp_id, pip.icmp_seqno, pip.icmp_str);
                }
                else
                {
                   fprintf(fp, "%s: %s %s <- %s\n          TTL=%d id=%d seq=%d %s\n",
                           pip.timestamp, pip.proto, pip.daddr, pip.saddr, pip.ttl,
                           pip.icmp_id, pip.icmp_seqno, pip.icmp_str);

                }
                break;
   }
}




void OpenAlertFile()
{
   char filename[STD_BUF];

   sprintf(filename, "%s/alert", pv.log_dir);

   if((alert = fopen(filename, "a")) == NULL)
   {
       fprintf(stderr, "ERROR in OpenAlertFile() => fopen() alert file: %s\n",
               strerror(errno));
       exit(1);
   }
} 
