/***************************************************************************
 *
 * File: MSTRING.C
 *
 * Purpose: Provide a variety of string functions not included in libc.  Makes
 *          up for the fact that the libstdc++ is hard to get reference 
 *          material on and I don't want to write any more non-portable c++
 *          code until I have solid references and libraries to use.
 *
 * History:
 *
 * Date:      Author:  Notes:
 * ---------- ------- ----------------------------------------------
 *  08/19/98    MFR    Initial coding begun
 *
 **************************************************************************/   
#include "mstring.h"

/*
int main()
{
   char *test="this is a test,	 mother fucker!";
   char **br; 
   int i;
   int toks;  


   br = mSplit(test, " \t", 6, &toks);

   for(i = 0; i < 6; i++)
   {
      printf("tok[%d] = %s\n", i, br[i]);
   }

   if(mContainsSubstr(test,"r!"))
   {
      printf("found substring!\n");
   }
   else
   {
      printf("doo dee doo...\n");
   }

   return 0;
}

*/
   
/****************************************************************
 *  
 *  Function: mSplit()
 * 
 *  Purpose: Splits a string into tokens non-destructively.
 *
 *  Parameters: 
 *      char *str => the string to be split
 *      char *sep => a string of token seperaters
 *      int max_strs => how many tokens should be returned
 *      int *toks => place to store the number of tokens found in str
 *
 *  Returns:
 *      2D char array with one token per "row" of the returned
 *      array.
 *
 ****************************************************************/
char **mSplit(char *str, char *sep, int max_strs, int *toks)
{
   char **retstr;    /* 2D array which is returned to caller */
   char *idx;        /* index pointer into str */
   char *end;        /* ptr to end of str */
   char *sep_end;    /* ptr to end of seperator string */
   char *sep_idx;    /* index ptr into seperator string */
   int len = 0;      /* length of current token string */
   int curr_str = 0; /* current index into the 2D return array */

   
   /* find the ends of the respective passed strings so our while() 
      loops know where to stop */
   sep_end = sep + strlen(sep);
   end = str + strlen(str);

   /* set our indexing pointers */
   sep_idx = sep;
   idx = str;

   /* alloc space for the return string, this is where the pointers to the
      tokens will be stored */
   retstr = (char **) malloc((sizeof(char **) * max_strs));

   /* loop thru each letter in the string being tokenized */
   while(idx < end)
   {
      /* loop thru each seperator string char */
      while(sep_idx < sep_end)
      {
         /* if the current string-indexed char matches the current
            seperator char... */
         if(*idx == *sep_idx)
         {
            /* if there's something to store... */
            if(len > 0)
            {
#ifdef DEBUG2
               printf("Allocating %d bytes for token ", len + 1);
               fflush(stdout);
#endif
               if(curr_str < max_strs)
               {
                  /* allocate space for the new token */
                  retstr[curr_str] = (char *) malloc((sizeof(char) * len) + 1);

                  /* make sure we got a good allocation */
                  if(retstr[curr_str] == NULL)  
                  {
                     fprintf(stderr, "msplit() got NULL substring malloc!\n");
                     exit(1);
                  }

                  /* copy the token into the return string array */
                  memcpy(retstr[curr_str], (idx - len), len);
                  retstr[curr_str][len] = 0; 
#ifdef DEBUG2
                  printf("%s\n", retstr[curr_str]);
                  fflush(stdout);
#endif
                  /* twiddle the necessary pointers and vars */
                  len = 0;
                  curr_str++;
                  idx++;
               }

               /* if we've gotten all the tokens requested, return the list */
               if(curr_str >= max_strs)
               {
                  *toks = curr_str + 1;
#ifdef DEBUG2
                  printf("mSplit got %d tokens!\n", *toks);
                  fflush(stdout);
#endif
                  return retstr;
               }
            }
            else  /* otherwise, the previous char was a seperator as well,
                     and we should just continue */
            {
               idx++;
               /* make sure to reset this so we test all the sep. chars */
               sep_idx = sep;
               len = 0;
            }
         }
         else
         {
            /* go to the next seperator */
            sep_idx++;
         }
      }

      sep_idx = sep;
      len++;
      idx++;
   }

   /* put the last string into the list */

   if(len > 0)
   {
#ifdef DEBUG2
      printf("Allocating %d bytes for token ", len + 1);
      fflush(stdout);
#endif

      retstr[curr_str] = (char *) malloc((sizeof(char) * len) + 1);

      if(retstr[curr_str] == NULL)
         printf("Got NULL back from substr malloc\n");

      memcpy(retstr[curr_str], (idx - len), len);
      retstr[curr_str][len] = 0; 

#ifdef DEBUG2
      printf("%s\n", retstr[curr_str]);
      fflush(stdout);
#endif
      *toks = curr_str + 1;
   }

#ifdef DEBUG2
   printf("mSplit got %d tokens!\n", *toks);
   fflush(stdout);
#endif   

   /* return the token list */
   return retstr;
}
      



/****************************************************************
 *
 *  Function: mContainsSubstr()
 *
 *  Purpose: Determines if a string contains a (non-regex) 
 *           substring.
 *
 *  Parameters:
 *      char *str => the string to be tested
 *      char *substr => the substr to look for
 *
 *  Returns:
 *      Integer value, 1 on success (str constains substr), 0 on
 *      failure (substr not in str)
 *
 ****************************************************************/
int mContainsSubstr(char *str, int str_len, char *substr, int substr_len)
{
   char *idx;      /* index ptr into the str array */
   char *sub_idx;  /* index ptr into the substr array */
   char *end;      /* ptr to end of str */
   char *sub_end;  /* ptr to end of substr */

   
   /* mark the end of the strs */
   end = (char *) (str + str_len);
   sub_end = (char *) (substr + substr_len);

   /* init the index ptrs */
   idx = str;
   sub_idx = substr;

   /* loop thru the substr a char at a time */
   while(sub_idx < sub_end)
   {
      /* if the current substr char matches the current 
         str char... */
      if(*sub_idx == *idx) 
      {
         /* go to the next char and continue checking */
         idx++;
         sub_idx++;

         
         if(sub_idx >= sub_end)
            return 1;

         if(idx >= end)
            return 0;
      }
      else
      {
         idx++;
         
         if(idx >= end)
            return 0;

         sub_idx = substr;
      }
   }

   /* if we loop all the way thru the substring, we've found what
      we're looking for */
   return 1;
}
     
   

